@@ -33,6 +33,8 @@
 #include <QPainter>
 #include <QFontMetrics>
 #include <QScrollBar>
+#include <QMenu>
+#include <QAction>
 
 #include "log.h"
 
@@ -42,6 +44,7 @@
 #include "quickfind.h"
 #include "quickfindpattern.h"
 #include "overview.h"
+#include "configuration.h"
 
 
 LineChunk::LineChunk( int first_col, int last_col, ChunkType type )
@@ -107,12 +110,15 @@ inline void LineDrawer::draw( QPainter& painter,
     QFontMetrics fm = painter.fontMetrics();
     const int fontHeight = fm.height();
     const int fontAscent = fm.ascent();
+    // For some reason on Qt 4.8.2 for Win, maxWidth() is wrong but the
+    // following give the right result, not sure why:
+    const int fontWidth = fm.width( QChar('a') );
 
     foreach ( Chunk chunk, list ) {
         // Draw each chunk
         // LOG(logDEBUG) << "Chunk: " << chunk.start() << " " << chunk.length();
         QString cutline = line.mid( chunk.start(), chunk.length() );
-        const int chunk_width = fm.width( cutline );
+        const int chunk_width = cutline.length() * fontWidth;
         painter.fillRect( xPos, yPos, chunk_width,
                 fontHeight, chunk.backColor() );
         painter.setPen( chunk.foreColor() );
@@ -200,7 +206,6 @@ AbstractLogView::AbstractLogView(const AbstractLogData* newLogData,
     overview_ = NULL;
 
     // Fonts
-    useFixedFont_ = false;
     charWidth_ = 1;
     charHeight_ = 1;
 
@@ -209,6 +214,9 @@ AbstractLogView::AbstractLogView(const AbstractLogData* newLogData,
 
     setAttribute( Qt::WA_StaticContents );  // Does it work?
 
+    // Init the popup menu
+    createMenu();
+
     // Signals
     connect( quickFindPattern_, SIGNAL( patternUpdated() ),
             this, SLOT ( handlePatternUpdated() ) );
@@ -247,6 +255,8 @@ void AbstractLogView::changeEvent( QEvent* changeEvent )
 
 void AbstractLogView::mousePressEvent( QMouseEvent* mouseEvent )
 {
+    static Configuration& config = Persistent<Configuration>( "settings" );
+
     // Selection implementation
     if ( mouseEvent->button() == Qt::LeftButton )
     {
@@ -272,6 +282,35 @@ void AbstractLogView::mousePressEvent( QMouseEvent* mouseEvent )
             selectionCurrentEndPos_ = selectionStartPos_;
         }
     }
+    else if ( mouseEvent->button() == Qt::RightButton )
+    {
+        // Prepare the popup depending on selection type
+        if ( selection_.isSingleLine() ) {
+            copyAction_->setText( "&Copy this line" );
+        }
+        else {
+            copyAction_->setText( "&Copy" );
+            copyAction_->setStatusTip( tr("Copy the selection") );
+        }
+
+        if ( selection_.isPortion() ) {
+            findNextAction_->setEnabled( true );
+            findPreviousAction_->setEnabled( true );
+            addToSearchAction_->setEnabled( true );
+        }
+        else {
+            findNextAction_->setEnabled( false );
+            findPreviousAction_->setEnabled( false );
+            addToSearchAction_->setEnabled( false );
+        }
+
+        // "Add to search" only makes sense in regexp mode
+        if ( config.mainRegexpType() != ExtendedRegexp )
+            addToSearchAction_->setEnabled( false );
+
+        // Display the popup (blocking)
+        popupMenu_->exec( QCursor::pos() );
+    }
 }
 
 void AbstractLogView::mouseMoveEvent( QMouseEvent* mouseEvent )
@@ -461,19 +500,11 @@ void AbstractLogView::keyPressEvent( QKeyEvent* keyEvent )
                     break;
                 case '*':
                     // Use the selected 'word' and search forward
-                    if ( selection_.isPortion() ) {
-                        emit changeQuickFind(
-                                selection_.getSelectedText( logData ) );
-                        searchNext();
-                    }
+                    findNextSelected();
                     break;
                 case '#':
                     // Use the selected 'word' and search backward
-                    if ( selection_.isPortion() ) {
-                        emit changeQuickFind(
-                                selection_.getSelectedText( logData ) );
-                        searchPrevious();
-                    }
+                    findPreviousSelected();
                     break;
                 default:
                     keyEvent->ignore();
@@ -534,7 +565,7 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
     {
         // Repaint the viewport
         QPainter painter( viewport() );
-        const int fontHeight = painter.fontMetrics().height();
+        const int fontHeight = charHeight_;
         const int fontAscent = painter.fontMetrics().ascent();
         const int nbCols = getNbVisibleCols();
         const QPalette& palette = viewport()->palette();
@@ -573,7 +604,8 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
             const int xPos = bulletLineX_ + 2;
 
             // string to print, cut to fit the length and position of the view
-            const QString cutLine = lines[i - firstLine].mid( firstCol, nbCols );
+            const QString line = lines[i - firstLine];
+            const QString cutLine = line.mid( firstCol, nbCols );
 
             if ( selection_.isLineSelected( i ) ) {
                 // Reverse the selected line
@@ -598,7 +630,7 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
             // Has the line got elements to be highlighted
             QList<QuickFindMatch> qfMatchList;
             bool isMatch =
-                quickFindPattern_->matchLine( cutLine, qfMatchList );
+                quickFindPattern_->matchLine( line, qfMatchList );
 
             if ( isSelection || isMatch ) {
                 // We use the LineDrawer and its chunks because the
@@ -609,14 +641,20 @@ void AbstractLogView::paintEvent( QPaintEvent* paintEvent )
                 QList<LineChunk> chunkList;
                 int column = 0; // Current column in line space
                 foreach( const QuickFindMatch match, qfMatchList ) {
-                    int start = match.startColumn();
+                    int start = match.startColumn() - firstCol;
+                    int end = start + match.length();
+                    // Ignore matches that are *completely* outside view area
+                    if ( ( start < 0 && end < 0 ) || start >= nbCols )
+                        continue;
                     if ( start > column )
                         chunkList << LineChunk( column, start - 1, LineChunk::Normal );
-                    column = start + match.length() - 1;
-                    chunkList << LineChunk( start, column, LineChunk::Highlighted );
+                    column = qMin( start + match.length() - 1, nbCols );
+                    chunkList << LineChunk( qMax( start, 0 ), column,
+                                            LineChunk::Highlighted );
                     column++;
                 }
-                chunkList << LineChunk( column, cutLine.length() - 1, LineChunk::Normal );
+                if ( column <= cutLine.length() - 1 )
+                    chunkList << LineChunk( column, cutLine.length() - 1, LineChunk::Normal );
 
                 // Then we add the selection if needed
                 QList<LineChunk> newChunkList;
@@ -772,6 +810,47 @@ void AbstractLogView::handlePatternUpdated()
     update();
 }
 
+// OR the current with the current search expression
+void AbstractLogView::addToSearch()
+{
+    if ( selection_.isPortion() ) {
+        LOG(logDEBUG) << "AbstractLogView::addToSearch()";
+        emit addToSearch( selection_.getSelectedText( logData ) );
+    }
+    else {
+        LOG(logERROR) << "AbstractLogView::addToSearch called for a wrong type of selection";
+    }
+}
+
+// Find next occurence of the selected text (*)
+void AbstractLogView::findNextSelected()
+{
+    // Use the selected 'word' and search forward
+    if ( selection_.isPortion() ) {
+        emit changeQuickFind(
+                selection_.getSelectedText( logData ) );
+        searchNext();
+    }
+}
+
+// Find next previous of the selected text (#)
+void AbstractLogView::findPreviousSelected()
+{
+    if ( selection_.isPortion() ) {
+        emit changeQuickFind(
+                selection_.getSelectedText( logData ) );
+        searchPrevious();
+    }
+}
+
+// Copy the selection to the clipboard
+void AbstractLogView::copy()
+{
+    static QClipboard* clipboard = QApplication::clipboard();
+
+    clipboard->setText( selection_.getSelectedText( logData ) );
+}
+
 //
 // Public functions
 //
@@ -815,19 +894,12 @@ void AbstractLogView::updateData()
 
 void AbstractLogView::updateDisplaySize()
 {
+    // Font is assumed to be mono-space (is restricted by options dialog)
     QFontMetrics fm = fontMetrics();
-
     charHeight_ = fm.height();
-    // Determine if the font is monospace (to enable various optimisations)
-    useFixedFont_ = true;
-    charWidth_ = fm.width('i');
-    for ( int i = 0x20; i < 0x7F; i++ )
-        if ( fm.width( QLatin1Char( i ) ) != charWidth_ ) {
-            useFixedFont_ = false;
-            break;
-        }
-    // FIXME: seems to be wrong on Qt/Mac
-    LOG(logDEBUG) << "useFixedFont_=" << useFixedFont_;
+    // For some reason on Qt 4.8.2 for Win, maxWidth() is wrong but the
+    // following give the right result, not sure why:
+    charWidth_ = fm.width( QChar('a') );
 
     // Calculate the index of the last line shown
     lastLine = qMin( logData->getNbLine(), firstLine + getNbVisibleLines() );
@@ -897,8 +969,7 @@ int AbstractLogView::getNbVisibleCols() const
 // Converts the mouse x, y coordinates to the line number in the file
 int AbstractLogView::convertCoordToLine(int yPos) const
 {
-    QFontMetrics fm = fontMetrics();
-    int line = firstLine + yPos/fm.height();
+    int line = firstLine + yPos / charHeight_;
 
     return line;
 }
@@ -913,21 +984,11 @@ QPoint AbstractLogView::convertCoordToFilePos( const QPoint& pos ) const
     if ( line < 0 )
         line = 0;
 
-    int column = 0;
-    // Determine column in screen space
-    QFontMetrics fm = fontMetrics();
+    // Determine column in screen space and convert it to file space
+    int column = firstCol + ( pos.x() - leftMarginPx_ ) / charWidth_;
+
     QString this_line = logData->getExpandedLineString( line );
     const int length = this_line.length();
-    for ( ; column < length; column++ ) {
-        if ( ( fm.width( this_line.mid(firstCol, column) )
-                    + leftMarginPx_ ) > pos.x() ) {
-            column--;
-            break;
-        }
-    }
-
-    // Now convert it to file space
-    column += firstCol;
 
     if ( column >= length )
         column = length - 1;
@@ -1095,3 +1156,39 @@ void AbstractLogView::updateGlobalSelection()
         clipboard->setText( selection_.getSelectedText( logData ),
                 QClipboard::Selection );
 }
+
+// Create the pop-up menu
+void AbstractLogView::createMenu()
+{
+    copyAction_ = new QAction( tr("&Copy"), this );
+    // No text as this action title depends on the type of selection
+    connect( copyAction_, SIGNAL(triggered()), this, SLOT(copy()) );
+
+    // For '#' and '*', shortcuts doesn't seem to work but
+    // at least it displays them in the menu, we manually handle those keys
+    // as keys event anyway (in keyPressEvent).
+    findNextAction_ = new QAction(tr("Find &next"), this);
+    findNextAction_->setShortcut( Qt::Key_Asterisk );
+    findNextAction_->setStatusTip( tr("Find the next occurence") );
+    connect( findNextAction_, SIGNAL(triggered()),
+            this, SLOT( findNextSelected() ) );
+
+    findPreviousAction_ = new QAction( tr("Find &previous"), this );
+    findPreviousAction_->setShortcut( tr("#")  );
+    findPreviousAction_->setStatusTip( tr("Find the previous occurence") );
+    connect( findPreviousAction_, SIGNAL(triggered()),
+            this, SLOT( findPreviousSelected() ) );
+
+    addToSearchAction_ = new QAction( tr("&Add to search"), this );
+    addToSearchAction_->setStatusTip(
+            tr("Add the selection to the current search") );
+    connect( addToSearchAction_, SIGNAL( triggered() ),
+            this, SLOT( addToSearch() ) );
+
+    popupMenu_ = new QMenu( this );
+    popupMenu_->addAction( copyAction_ );
+    popupMenu_->addSeparator();
+    popupMenu_->addAction( findNextAction_ );
+    popupMenu_->addAction( findPreviousAction_ );
+    popupMenu_->addAction( addToSearchAction_ );
+}
