@@ -30,6 +30,9 @@
 #include <QLineEdit>
 #include <QFileInfo>
 #include <QKeyEvent>
+#include <QStandardItemModel>
+#include <QHeaderView>
+#include <QListView>
 
 #include "crawlerwidget.h"
 
@@ -68,6 +71,61 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
 
     savedSearches = searches;
 
+    // Construct the visibility button
+    visibilityModel_ = new QStandardItemModel( this );
+
+    QStandardItem *marksAndMatchesItem = new QStandardItem( tr( "Marks and matches" ) );
+    QPixmap marksAndMatchesPixmap( 16, 10 );
+    marksAndMatchesPixmap.fill( Qt::gray );
+    marksAndMatchesItem->setIcon( QIcon( marksAndMatchesPixmap ) );
+    marksAndMatchesItem->setData( FilteredView::MarksAndMatches );
+    visibilityModel_->appendRow( marksAndMatchesItem );
+
+    QStandardItem *marksItem = new QStandardItem( tr( "Marks" ) );
+    QPixmap marksPixmap( 16, 10 );
+    marksPixmap.fill( Qt::blue );
+    marksItem->setIcon( QIcon( marksPixmap ) );
+    marksItem->setData( FilteredView::MarksOnly );
+    visibilityModel_->appendRow( marksItem );
+
+    QStandardItem *matchesItem = new QStandardItem( tr( "Matches" ) );
+    QPixmap matchesPixmap( 16, 10 );
+    matchesPixmap.fill( Qt::red );
+    matchesItem->setIcon( QIcon( matchesPixmap ) );
+    matchesItem->setData( FilteredView::MatchesOnly );
+    visibilityModel_->appendRow( matchesItem );
+
+    QListView *visibilityView = new QListView( this );
+    visibilityView->setMovement( QListView::Static );
+    visibilityView->setMinimumWidth( 170 ); // Only needed with custom style-sheet
+
+    visibilityBox = new QComboBox();
+    visibilityBox->setModel( visibilityModel_ );
+    visibilityBox->setView( visibilityView );
+
+    // Select "Marks and matches" by default (same default as the filtered view)
+    visibilityBox->setCurrentIndex( 0 );
+
+    // TODO: Maybe there is some way to set the popup width to be
+    // sized-to-content (as it is when the stylesheet is not overriden) in the
+    // stylesheet as opposed to setting a hard min-width on the view above.
+    visibilityBox->setStyleSheet( " \
+        QComboBox:on {\
+            padding: 1px 2px 1px 6px;\
+            width: 19px;\
+        } \
+        QComboBox:!on {\
+            padding: 1px 2px 1px 7px;\
+            width: 19px;\
+            height: 16px;\
+            border: 1px solid gray;\
+        } \
+        QComboBox::drop-down::down-arrow {\
+            width: 0px;\
+            border-width: 0px;\
+        } \
+" );
+
     // Construct the Search Info line
     searchInfoLine = new InfoLine();
     searchInfoLine->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
@@ -110,6 +168,7 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
     searchButton->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum ) );
 
     QHBoxLayout* searchInfoLineLayout = new QHBoxLayout;
+    searchInfoLineLayout->addWidget( visibilityBox );
     searchInfoLineLayout->addWidget( searchInfoLine );
     searchInfoLineLayout->addWidget( ignoreCaseCheck );
     searchInfoLineLayout->addWidget( searchRefreshCheck );
@@ -143,6 +202,9 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
     connect(stopButton, SIGNAL( clicked() ),
             this, SLOT( stopSearch() ) );
 
+    connect(visibilityBox, SIGNAL( currentIndexChanged( int ) ),
+            this, SLOT( changeFilteredViewVisibility( int ) ) );
+
     connect(logMainView, SIGNAL( newSelection( int ) ),
             logMainView, SLOT( update() ) );
     connect(filteredView, SIGNAL( newSelection( int ) ),
@@ -156,6 +218,11 @@ CrawlerWidget::CrawlerWidget(SavedSearches* searches, QWidget *parent)
     connect(filteredView, SIGNAL( markLine( qint64 ) ),
             this, SLOT( markLineFromFiltered( qint64 ) ) );
 
+    connect(logMainView, SIGNAL( addToSearch( const QString& ) ),
+            this, SLOT( addToSearch( const QString& ) ) );
+    connect(filteredView, SIGNAL( addToSearch( const QString& ) ),
+            this, SLOT( addToSearch( const QString& ) ) );
+
     // Follow option (up and down)
     connect(this, SIGNAL( followSet( bool ) ),
             logMainView, SLOT( followSet( bool ) ) );
@@ -388,6 +455,13 @@ void CrawlerWidget::applyConfiguration()
 
     LOG(logDEBUG) << "CrawlerWidget::applyConfiguration";
 
+    // Whatever font we use, we should NOT use kerning
+    font.setKerning( false );
+    font.setFixedPitch( true );
+#if QT_VERSION > 0x040700
+    // Necessary on systems doing subpixel positionning (e.g. Ubuntu 12.04)
+    font.setStyleStrategy( QFont::ForceIntegerMetrics );
+#endif
     logMainView->setFont(font);
     filteredView->setFont(font);
 
@@ -512,6 +586,32 @@ void CrawlerWidget::searchTextChangeHandler()
     printSearchInfoMessage( logFilteredData_->getNbMatches() );
 }
 
+void CrawlerWidget::changeFilteredViewVisibility( int index )
+{
+    QStandardItem* item = visibilityModel_->item( index );
+    FilteredView::Visibility visibility =
+        static_cast< FilteredView::Visibility>( item->data().toInt() );
+
+    filteredView->setVisibility( visibility );
+}
+
+void CrawlerWidget::addToSearch( const QString& string )
+{
+    QString text = searchLineEdit->currentText();
+
+    if ( text.isEmpty() )
+        text = string;
+    else {
+        // Escape the regexp chars from the string before adding it.
+        text += ( '|' + QRegExp::escape( string ) );
+    }
+
+    searchLineEdit->setEditText( text );
+
+    // Set the focus to lineEdit so that the user can press 'Return' immediately
+    searchLineEdit->lineEdit()->setFocus();
+}
+
 //
 // Private functions
 //
@@ -535,7 +635,7 @@ void CrawlerWidget::replaceCurrentSearch( const QString& searchText )
     if ( !searchText.isEmpty() ) {
         // Determine the type of regexp depending on the config
         QRegExp::PatternSyntax syntax;
-        Configuration& config = Persistent<Configuration>( "settings" );
+        static Configuration& config = Persistent<Configuration>( "settings" );
         switch ( config.mainRegexpType() ) {
             case Wildcard:
                 syntax = QRegExp::Wildcard;
