@@ -35,6 +35,7 @@
 #include "optionsdialog.h"
 #include "persistentinfo.h"
 #include "savedsearches.h"
+#include "menuactiontooltipbehavior.h"
 
 MainWindow::MainWindow() :
     recentFiles( Persistent<RecentFiles>( "recentFiles" ) ), mainIcon_()
@@ -135,7 +136,7 @@ void MainWindow::createActions()
 
     copyAction = new QAction(tr("&Copy"), this);
     copyAction->setShortcut(QKeySequence::Copy);
-    copyAction->setStatusTip(tr("Copy the selected line"));
+    copyAction->setStatusTip(tr("Copy the selection"));
     connect( copyAction, SIGNAL(triggered()), this, SLOT(copy()) );
 
     findAction = new QAction(tr("&Find..."), this);
@@ -144,7 +145,7 @@ void MainWindow::createActions()
     connect( findAction, SIGNAL(triggered()),
             this, SLOT( find() ) );
 
-    overviewVisibleAction = new QAction( tr("Matches overview"), this );
+    overviewVisibleAction = new QAction( tr("Matches &overview"), this );
     overviewVisibleAction->setCheckable( true );
     overviewVisibleAction->setChecked( config.isOverviewVisible() );
     connect( overviewVisibleAction, SIGNAL( toggled( bool ) ),
@@ -157,6 +158,7 @@ void MainWindow::createActions()
             this, SIGNAL(followSet( bool )) );
 
     reloadAction = new QAction( tr("&Reload"), this );
+    reloadAction->setShortcut(QKeySequence::Refresh);
     reloadAction->setIcon( QIcon(":/images/reload16.png") );
     connect( reloadAction, SIGNAL(triggered()), this, SLOT(reload()) );
 
@@ -187,8 +189,11 @@ void MainWindow::createMenus()
     fileMenu = menuBar()->addMenu( tr("&File") );
     fileMenu->addAction( openAction );
     fileMenu->addSeparator();
-    for (int i = 0; i < MaxRecentFiles; ++i)
+    for (int i = 0; i < MaxRecentFiles; ++i) {
         fileMenu->addAction( recentFileActions[i] );
+        recentFileActionBehaviors[i] =
+            new MenuActionToolTipBehavior(recentFileActions[i], fileMenu, this);
+    }
     fileMenu->addSeparator();
     fileMenu->addAction( exitAction );
 
@@ -268,7 +273,7 @@ void MainWindow::openRecentFile()
 // Copy the currently selected line into the clipboard
 void MainWindow::copy()
 {
-    QClipboard* clipboard = QApplication::clipboard();
+    static QClipboard* clipboard = QApplication::clipboard();
 
     clipboard->setText( crawlerWidget->getSelectedText() );
 
@@ -320,7 +325,7 @@ void MainWindow::about()
 #ifdef GLOGG_COMMIT
                 "<p>Built " GLOGG_DATE " from " GLOGG_COMMIT
 #endif
-                "<p>Copyright &copy; 2009, 2010, 2011 Nicolas Bonnefon and other contributors"
+                "<p>Copyright &copy; 2009, 2010, 2011 2012 Nicolas Bonnefon and other contributors"
                 "<p>You may modify and redistribute the program under the terms of the GPL (version 3 or later)." ) );
 }
 
@@ -500,6 +505,7 @@ void MainWindow::updateRecentFileActions()
         if ( j < recent_files.count() ) {
             QString text = tr("&%1 %2").arg(j + 1).arg(strippedName(recent_files[j]));
             recentFileActions[j]->setText( text );
+            recentFileActions[j]->setToolTip( recent_files[j] );
             recentFileActions[j]->setData( recent_files[j] );
             recentFileActions[j]->setVisible( true );
         }
